@@ -22,9 +22,10 @@ use crate::comms::*;
 use crate::spec::*;
 use crate::error::*;
 use crate::lint::*;
+use crate::time::*;
 
 use crate::crypto::{EncryptedPrivateKey, PrivateSignKey};
-use crate::{crypto::EncryptKey, session::{Session, SessionProperty}};
+use crate::{crypto::EncryptKey, session::{AteSession, AteSessionProperty}};
 
 #[derive(Debug)]
 pub(crate) struct DioState
@@ -116,9 +117,10 @@ where Self: Send + Sync
 {
     pub(super) multi: ChainMultiUser,
     pub(super) state: DioState,
-    pub(super) session: &'a Session,
-    pub(super) scope: Scope,
+    pub(super) session: &'a AteSession,
+    pub(super) scope: TransactionScope,
     pub(super) conversation: Option<Arc<ConversationSession>>,
+    pub(super) time: Arc<TimeKeeper>,
 }
 
 impl<'a> Dio<'a>
@@ -282,28 +284,81 @@ impl<'a> Dio<'a>
     pub async fn children<D>(&mut self, parent_id: PrimaryKey, collection_id: u64) -> Result<Vec<Dao<D>>, LoadError>
     where D: Serialize + DeserializeOwned + Clone + Send + Sync,
     {
-        self.children_ext(parent_id, collection_id, false).await
+        self.children_ext(parent_id, collection_id, false, false).await
     }
 
-    pub async fn children_ext<D>(&mut self, parent_id: PrimaryKey, collection_id: u64, allow_missing_keys: bool) -> Result<Vec<Dao<D>>, LoadError>
+    pub async fn children_ext<D>(&mut self, parent_id: PrimaryKey, collection_id: u64, allow_missing_keys: bool, allow_serialization_error: bool) -> Result<Vec<Dao<D>>, LoadError>
     where D: Serialize + DeserializeOwned + Clone + Send + Sync,
     {
         // Build the secondary index key
-        let key = MetaCollection {
+        let collection_key = MetaCollection {
             parent_id,
             collection_id,
         };
 
+        // Build a list of keys
+        let keys = match self.multi.lookup_secondary_raw(&collection_key).await {
+            Some(a) => a,
+            None => return Ok(Vec::new())
+        };
+
+        // Load all the objects
+        let mut ret: Vec<Dao<D>> = self.load_many_ext(keys.into_iter(), allow_missing_keys, allow_serialization_error).await?;
+
+        // Build an already loaded list
+        let mut already = FxHashSet::default();
+        for a in ret.iter() {
+            already.insert(a.key().clone());
+        }
+
+        // Now we search the secondary local index so any objects we have
+        // added in this transaction scope are returned
+        let state = &self.state;
+        if let Some(vec) = state.cache_store_secondary.get_vec(&collection_key) {
+            for a in vec {
+                // This is an OR of two lists so its likely that the object
+                // may already be in the return list
+                if already.contains(a) {
+                    continue;
+                }
+                if state.deleted.contains(a) {
+                    continue;
+                }
+
+                // If its still locked then that is a problem
+                if state.is_locked(a) {
+                    return Result::Err(LoadError::ObjectStillLocked(a.clone()));
+                }
+
+                if let Some(dao) = state.cache_store_primary.get(a) {
+                    let row = Row::from_row_data(dao.deref())?;
+    
+                    already.insert(row.key.clone());
+                    ret.push(Dao::new(DaoEthereal::new(row)));
+                }
+            }
+        }
+
+        Ok(ret)
+    }
+
+    pub async fn load_many<D>(&mut self, keys: impl Iterator<Item=PrimaryKey>) -> Result<Vec<Dao<D>>, LoadError>
+    where D: Serialize + DeserializeOwned + Clone + Send + Sync,
+    {
+        self.load_many_ext(keys, false, false).await
+    }
+
+    pub async fn load_many_ext<D>(&mut self, keys: impl Iterator<Item=PrimaryKey>, allow_missing_keys: bool, allow_serialization_error: bool) -> Result<Vec<Dao<D>>, LoadError>
+    where D: Serialize + DeserializeOwned + Clone + Send + Sync,
+    {
         // This is the main return list
         let mut already = FxHashSet::default();
         let mut ret = Vec::new();
 
         // We either find existing objects in the cache or build a list of objects to load
         let mut to_load = Vec::new();
-        for key in match self.multi.lookup_secondary_raw(&key).await {
-            Some(a) => a,
-            None => return Ok(Vec::new())
-        } {
+        for key in keys
+        {
             {
                 let state = &self.state;
                 if state.is_locked(&key) {
@@ -380,61 +435,47 @@ impl<'a> Dio<'a>
                 None => { continue; },
             };
 
-            let row = Row::from_event(&evt.data, evt.leaf.created, evt.leaf.updated)?;
+            let row = match Row::from_event(&evt.data, evt.leaf.created, evt.leaf.updated) {
+                Ok(a) => a,
+                Err(err) => {
+                    if allow_serialization_error {
+                        debug!("Serialization error {} - ignoring row", err);
+                        continue;
+                    }
+                    return Err(LoadError::SerializationError(err));
+                }
+            };
             state.cache_load.insert(row.key.clone(), (Arc::new(evt.data), evt.leaf));
 
             already.insert(row.key.clone());
             ret.push(Dao::new(DaoEthereal::new(row)));
         }
 
-        // Now we search the secondary local index so any objects we have
-        // added in this transaction scope are returned
-        let state = &self.state;
-        if let Some(vec) = state.cache_store_secondary.get_vec(&key) {
-            for a in vec {
-                // This is an OR of two lists so its likely that the object
-                // may already be in the return list
-                if already.contains(a) {
-                    continue;
-                }
-                if state.deleted.contains(a) {
-                    continue;
-                }
-
-                // If its still locked then that is a problem
-                if state.is_locked(a) {
-                    return Result::Err(LoadError::ObjectStillLocked(a.clone()));
-                }
-
-                if let Some(dao) = state.cache_store_primary.get(a) {
-                    let row = Row::from_row_data(dao.deref())?;
-    
-                    already.insert(row.key.clone());
-                    ret.push(Dao::new(DaoEthereal::new(row)));
-                }
-            }
-        }
-
         Ok(ret)
     }
+
+    pub fn session(&'a self) -> &'a AteSession {
+        self.session
+    }
 }
 
 impl Chain
 {
     #[allow(dead_code)]
-    pub async fn dio<'a>(&'a self, session: &'a Session) -> Dio<'a> {
-        self.dio_ext(session, Scope::Local).await
+    pub async fn dio<'a>(&'a self, session: &'a AteSession) -> Dio<'a> {
+        self.dio_ext(session, TransactionScope::Local).await
     }
 
     #[allow(dead_code)]
-    pub async fn dio_ext<'a>(&'a self, session: &'a Session, scope: Scope) -> Dio<'a> {
+    pub async fn dio_ext<'a>(&'a self, session: &'a AteSession, scope: TransactionScope) -> Dio<'a> {
         let multi = self.multi().await;
         Dio {
             state: DioState::new(),
             multi,
-            session: session,
+            session,
             scope,
-            conversation: self.pipe.conversation().await
+            conversation: self.pipe.conversation().await,
+            time: Arc::clone(&self.time),
         }
     }
 }
@@ -496,6 +537,7 @@ impl<'a> Dio<'a>
 
                 // Build a new clean metadata header
                 let mut meta = Metadata::for_data(row.key);
+                meta.core.push(CoreMetadata::Timestamp(self.time.current_timestamp()?));
                 if row.auth.is_relevant() {
                     meta.core.push(CoreMetadata::Authorization(row.auth.clone()));
                 }
@@ -524,7 +566,9 @@ impl<'a> Dio<'a>
                         }
                     });
                     if let Some(parent) = meta.get_parent() {
-                        trans_meta.parents.insert(key, parent.clone());
+                        if parent.vec.parent_id != key {
+                            trans_meta.parents.insert(key, parent.clone());
+                        }
                     }
                 }
                 
@@ -543,6 +587,7 @@ impl<'a> Dio<'a>
             // Build events that will represent tombstones on all these records (they will be sent after the writes)
             for key in state.deleted.drain() {
                 let mut meta = Metadata::default();
+                meta.core.push(CoreMetadata::Timestamp(self.time.current_timestamp()?));
                 meta.core.push(CoreMetadata::Authorization(MetaAuthorization {
                     read: ReadOption::Everyone(None),
                     write: WriteOption::Nobody,
@@ -572,6 +617,7 @@ impl<'a> Dio<'a>
                     header: evt.as_header()?,
                 });
             }
+
             let meta = multi_lock.metadata_lint_many(&lints, &self.session, self.conversation.as_ref())?;
 
             // If it has data then insert it at the front of these events
