@@ -29,7 +29,8 @@ use crate::flow::OpenAction;
 use crate::spec::SerializationFormat;
 use crate::repository::ChainRepository;
 use crate::comms::TxDirection;
-use crate::crypto::Hash;
+use crate::crypto::AteHash;
+use crate::time::ChainTimestamp;
 
 pub struct MeshRoot<F>
 where Self: ChainRepository,
@@ -100,6 +101,7 @@ where F: OpenFlow + 'static
     {
         let mut node_cfg = NodeConfig::new(cfg_ate.wire_format)
             .wire_encryption(cfg_ate.wire_encryption)
+            .timeout(cfg_ate.connect_timeout)
             .buffer_size(cfg_ate.buffer_size_server);
         let mut listen_ports = listen_addrs
             .iter()
@@ -246,7 +248,7 @@ struct OpenContext<'a>
 async fn open_internal<'a, F>(root: Arc<MeshRoot<F>>, key: ChainKey, context: Option<OpenContext<'a>>) -> Result<Arc<Chain>, ChainCreationError>
 where F: OpenFlow + 'static
 {
-    debug!("open {}", key.to_string());
+    debug!("open_internal {}", key.to_string());
 
     {
         let chains = root.chains.lock();
@@ -270,7 +272,7 @@ where F: OpenFlow + 'static
     }
 
     // Create a chain builder
-    let mut builder = ChainOfTrustBuilder::new(&root.cfg_ate)
+    let mut builder = ChainBuilder::new(&root.cfg_ate)
         .await;
 
     // Add a pipe that will broadcast message to the connected clients
@@ -288,6 +290,7 @@ where F: OpenFlow + 'static
     }
 
     // Create the chain using the chain flow builder
+    debug!("open_flow: {}", std::any::type_name::<F>());
     let new_chain = match chain_builder_flow.open(builder, &key).await? {
         OpenAction::PrivateChain { chain, session} => {
             if let Some(ctx) = &context {
@@ -350,9 +353,9 @@ async fn inbox_event(
     let commit = commit.clone();
     
     // Feed the events into the chain of trust
-    let evts = MessageEvent::convert_from(evts);
+    let evts = MessageEvent::convert_from(evts.into_iter());
     let ret = chain.pipe.feed(Transaction {
-        scope: Scope::None,
+        scope: TransactionScope::None,
         transmit: false,
         events: evts,
         conversation: Some(Arc::clone(&context.conversation)),
@@ -431,6 +434,7 @@ async fn inbox_unlock(
 async fn inbox_subscribe<F>(
     root: Arc<MeshRoot<F>>,
     chain_key: ChainKey,
+    from: ChainTimestamp,
     reply_at: Option<&mpsc::Sender<PacketData>>,
     session_context: Arc<SessionContext>,
     wire_format: SerializationFormat,
@@ -484,21 +488,18 @@ where F: OpenFlow + 'static
         session_context.group.store(chain.key().hash64(), std::sync::atomic::Ordering::Relaxed);
     }
 
-    // Grab the first sample point (if we have no history then we are done)
-    let guard = chain.multi().await;
-    let guard = guard.inside_async.read().await;
-    match guard.range(..).next() {
-        Some(a) => {
-            PacketData::reply_at(reply_at, wire_format, Message::SampleRightOf(a.event_hash)).await?;
-        },
-        None => {
-            if let Some(reply_at) = reply_at {
-                stream_empty_history(chain, None, reply_at.clone(), wire_format).await?;
-            } else {
-                debug!("no reply address for this subscribe");
-            }
-        }
-    };
+    // Stream the data back to the client
+    if let Some(reply_at) = reply_at {
+        debug!("inbox: starting the streaming process");
+        tokio::spawn(stream_history_range(
+            Arc::clone(&chain), 
+            from.., 
+            reply_at.clone(),
+            wire_format,
+        ));
+    } else {
+        debug!("no reply address for this subscribe");
+    }
 
     Ok(())
 }
@@ -517,84 +518,6 @@ where F: OpenFlow + 'static
     Ok(())
 }
 
-async fn inbox_samples_of_history(
-    context: Arc<SessionContext>,
-    pivot: Hash,
-    history_sample: Vec<Hash>,
-    reply_at: Option<&mpsc::Sender<PacketData>>,
-    wire_format: SerializationFormat,
-)
--> Result<(), CommsError>
-{
-    debug!("inbox: inbox_samples_of_history (pivot={}, samples={})", pivot, history_sample.len());
-
-    // We have got as close as we can - lets start the history sending process now
-    if let Some(reply_at) = reply_at
-    {
-        // Get the chain
-        let chain = match context.inside.lock().chain.clone() {
-            Some(a) => a,
-            None => { return Ok(()); }
-        };
-
-        // If no samples were supplied at all then just send the entire history as they dont have
-        // anything at all (or we lost sync)
-        if history_sample.len() <= 0 {
-            debug!("inbox: streaming the entire history");
-            tokio::spawn(stream_history_range(
-                Arc::clone(&chain), 
-                .., 
-                reply_at.clone(),
-                wire_format,
-            ));
-            return Ok(())
-        }
-
-        // See if the pivot point gets closer to the end of the history - if it does not then
-        // keep taking samples
-        if let Some(hash) = locate_pivot_within_history(&chain, history_sample).await {
-            if hash != pivot {
-                debug!("inbox: pivot is still moving forward (pivot={})", hash);
-                PacketData::reply_at(Some(&reply_at), wire_format, Message::SampleRightOf(hash)).await?;
-                return Ok(());
-            }
-        }
-
-        // Move to the right of the pivot point
-        let next = {
-            let multi = chain.multi().await;
-            let guard = multi.inside_async.read().await;
-            let mut iter = guard.range(pivot..).map(|e| e.event_hash);
-            iter.next();
-            iter.next()
-        };
-        let next = match next {
-            Some(a) => {
-                debug!("inbox: pivot has settled in the middle (pivot={})", pivot);
-                a
-            },
-            None => {
-                debug!("inbox: pivot has settled at the eof");
-                stream_empty_history(Arc::clone(&chain), Some(pivot), reply_at.clone(), wire_format).await?;
-                return Ok(())
-            }
-        };
-
-        // Stream the data back to the client
-        debug!("inbox: starting the streaming process");
-        tokio::spawn(stream_history_range(
-            Arc::clone(&chain), 
-            next.., 
-            reply_at.clone(),
-            wire_format,
-        ));
-    } else {
-        debug!("no reply address for this streaming of data");
-    }
-
-    Ok(())
-}
-
 async fn inbox_packet<F>(
     root: Arc<MeshRoot<F>>,
     pck: PacketWithContext<Message, SessionContext>,
@@ -614,10 +537,8 @@ where F: OpenFlow + 'static
     let reply_at = reply_at_owner.as_ref();
     
     match pck.msg {
-        Message::Subscribe { chain_key }
-            => inbox_subscribe(root, chain_key, reply_at, context, wire_format, tx).await,
-        Message::SamplesOfHistory { pivot, samples }
-            => inbox_samples_of_history(context, pivot, samples, reply_at, wire_format).await,
+        Message::Subscribe { chain_key, from }
+            => inbox_subscribe(root, chain_key, from, reply_at, context, wire_format, tx).await,
         Message::Events { commit, evts }
             => inbox_event(reply_at, context, commit, evts, tx, pck_data).await,
         Message::Lock { key }
