@@ -2,18 +2,26 @@
 use log::{error, info, warn, debug};
 
 mod api;
+#[cfg(feature = "local_fs")]
+mod file_localfs;
+mod file_memdb;
 mod file;
 mod flags;
 mod flip;
+mod magic;
+#[cfg(feature = "local_fs")]
+mod appender;
 mod loader;
-mod reader;
+#[cfg(feature = "local_fs")]
+mod archive;
 mod core;
-mod seeker;
 mod test;
 
-static REDO_MAGIC: &'static [u8; 4] = b"REDO";
-
 pub use flags::OpenFlags;
 pub use loader::RedoLogLoader;
 pub use self::core::RedoLog;
-pub use api::LogWritable;
\ No newline at end of file
+pub use api::LogWritable;
+
+pub(crate) use api::LogLookup;
+
+pub use file::*;
\ No newline at end of file
